#include "config.h"
#include <gegl-plugin.h>
void gegl_op_aces_rrt_register_type(GTypeModule *module);
void gegl_op_alpha_inpaint_register_type(GTypeModule *module);
void gegl_op_bilateral_filter_fast_register_type(GTypeModule *module);
void gegl_op_boxblur_1d_register_type(GTypeModule *module);
void gegl_op_boxblur_register_type(GTypeModule *module);
void gegl_op_connected_components_register_type(GTypeModule *module);
void gegl_op_demosaic_bimedian_register_type(GTypeModule *module);
void gegl_op_demosaic_simple_register_type(GTypeModule *module);
void gegl_op_ditto_register_type(GTypeModule *module);
void gegl_op_band_tune_register_type(GTypeModule *module);
void gegl_op_gcr_register_type(GTypeModule *module);
void gegl_op_gradient_map_register_type(GTypeModule *module);
void gegl_op_integral_image_register_type(GTypeModule *module);
void gegl_op_rawbayer_load_register_type(GTypeModule *module);
void gegl_op_segment_kmeans_register_type(GTypeModule *module);
void gegl_op_shuffle_search_register_type(GTypeModule *module);
void gegl_op_selective_hue_saturation_register_type(GTypeModule *module);
static const GeglModuleInfo modinfo = {
GEGL_MODULE_ABI_VERSION
};

G_MODULE_EXPORT const GeglModuleInfo * gegl_module_query (GTypeModule *module);
G_MODULE_EXPORT gboolean gegl_module_register (GTypeModule *module);

const GeglModuleInfo *
gegl_module_query (GTypeModule *module)
{
  return &modinfo;
}

gboolean
gegl_module_register (GTypeModule *module)
{
  gegl_op_aces_rrt_register_type(module);
  gegl_op_alpha_inpaint_register_type(module);
  gegl_op_bilateral_filter_fast_register_type(module);
  gegl_op_boxblur_1d_register_type(module);
  gegl_op_boxblur_register_type(module);
  gegl_op_connected_components_register_type(module);
  gegl_op_demosaic_bimedian_register_type(module);
  gegl_op_demosaic_simple_register_type(module);
  gegl_op_ditto_register_type(module);
  gegl_op_band_tune_register_type(module);
  gegl_op_gcr_register_type(module);
  gegl_op_gradient_map_register_type(module);
  gegl_op_integral_image_register_type(module);
  gegl_op_rawbayer_load_register_type(module);
  gegl_op_segment_kmeans_register_type(module);
  gegl_op_shuffle_search_register_type(module);
  gegl_op_selective_hue_saturation_register_type(module);
  return TRUE;
}
